summaryrefslogtreecommitdiff
path: root/src/libcamera/pipeline_handler.cpp
diff options
context:
space:
mode:
Diffstat (limited to 'src/libcamera/pipeline_handler.cpp')
-rw-r--r--src/libcamera/pipeline_handler.cpp30
1 files changed, 28 insertions, 2 deletions
diff --git a/src/libcamera/pipeline_handler.cpp b/src/libcamera/pipeline_handler.cpp
index 2374c289..92b5e3ab 100644
--- a/src/libcamera/pipeline_handler.cpp
+++ b/src/libcamera/pipeline_handler.cpp
@@ -267,8 +267,7 @@ void PipelineHandler::unlock()
*/
/**
* \fn PipelineHandler::stop()
* \brief Stop capturing from all running streams
* \param[in] camera The camera to stop
*
* This function stops capturing and processing requests immediately. All
@@ -276,6 +275,33 @@ void PipelineHandler::unlock()
*
* \context This function is called from the CameraManager thread.
*/
+void PipelineHandler::stop(Camera *camera)
+{
+ /* Stop the pipeline handler and let the queued requests complete. */
+ stopDevice(camera);
+
+ /* Cancel and signal as complete all waiting requests. */
+ while (!waitingRequests_.empty()) {
+ Request *request = waitingRequests_.front();
+ waitingRequests_.pop();
+
+ request->_d()->cancel();
+ completeRequest(request);
+ }
+
+ /* Make sure no requests are pending. */
+ Camera::Private *data = camera->_d();
+ ASSERT(data->queuedRequests_.empty());
+}
+
+/**
+ * \fn PipelineHandler::stopDevice()
+ * \brief Stop capturing from all running streams
+ * \param[in] camera The camera to stop
+ *
+ * This function stops capturing and processing requests immediately. All
+ * pending requests are cancelled and complete immediately in an error state.
+ */
/**
* \brief Determine if the camera has any requests pending
@@ -267,8 +267,7 @@ void PipelineHandler::unlock()
*/
/**
- * \fn PipelineHandler::stop()
- * \brief Stop capturing from all running streams
+ * \brief Stop capturing from all running streams and cancel pending requests
* \param[in] camera The camera to stop
*